# -*- coding: utf-8 -*-
import unittest

from a3exception import errors


class T(unittest.TestCase):
    def test__error(self):
        message = "message"
        cause = "cause"
        error = errors.Error(error_type=errors.ErrorType.ClientSideError, message=message, cause=cause)

        self.assertEqual(str(error), message)
        self.assertEqual(repr(error), cause)

    def test__client_errors(self):
        text = "text"
        self.assertEqual(errors.AuthenticationFailedError(message=text).message, text)
        self.assertEqual(errors.ValidationError(message=text).message, text)
        self.assertEqual(errors.NotFoundError(message=text).message, text)
        self.assertEqual(errors.InvalidTokenError(message=text).message, text)
        self.assertEqual(errors.ForbiddenError(message=text).message, text)
        self.assertEqual(errors.NotAvailableError(message=text).message, text)
        self.assertEqual(errors.ClientKnownError(message=text).message, text)

    def test__server_errors(self):
        text = "text"
        self.assertEqual(errors.ServerKnownError(cause=text).cause, text)
        self.assertEqual(errors.ServerUnknownError(cause=text).cause, text)



# -*- coding: utf-8 -*-
from typing import Type, Dict
from a3exception import errors
from a3py.practical.dynamic import find_all_subclasses


class DynamicErrorFactory:
    _has_build_cache = False
    _cache_errors: Dict[str, Type[errors.Error]] = dict()

    @classmethod
    def add_custom_error_cls(cls, err_cls: Type[errors.Error]):
        status = err_cls.__name__
        cls._cache_errors[status] = err_cls

    @classmethod
    def build_error_by_status(cls, status: str, **kwargs) -> errors.Error:
        if not cls._has_build_cache:
            for err_cls in find_all_subclasses(errors, errors.Error):
                cls.add_custom_error_cls(err_cls=err_cls)
            cls._has_build_cache = True

        err_cls = cls._cache_errors.get(status)
        assert err_cls is not None, (
            f"This custom error has not been added `{status}`, please use `add_custom_error_cls` to add it."
        )

        params = {"message": None, "cause": None, "detail": None}
        params.update(kwargs)
        error_instance = err_cls(**params)

        return error_instance



# -*- coding: utf-8 -*-
import unittest

from a3exception.dynamic_error_factory import DynamicErrorFactory
from a3exception import errors


class T(unittest.TestCase):
    def test__build_error_by_status__success(self):
        error = DynamicErrorFactory.build_error_by_status(status=errors.NotFoundError.__name__)
        self.assertTrue(isinstance(error, errors.NotFoundError))

        error = DynamicErrorFactory.build_error_by_status(status=errors.NotAvailableError.__name__)
        self.assertTrue(isinstance(error, errors.NotAvailableError))



# -*- coding: utf-8 -*-


class ErrorType:
    ServerSideError = "ServerSideError"
    ClientSideError = "ClientSideError"


class Error(Exception):
    error_type = None
    message = None

    def __init__(
        self,
        message: str | None = None,
        cause: str | None = None,
        detail: dict | None = None,
        status: str | None = None,
        error_type: str | None = None,
    ):
        self.message = message or self.message
        self.cause = cause
        self.detail = detail
        self.status = status or self.__class__.__name__
        self.error_type = error_type or self.error_type
        assert self.error_type is not None, "error_type must be set."

    def __str__(self):
        return self.message

    def __repr__(self):
        return self.cause


class AuthenticationFailedError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


class ValidationError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


class NotFoundError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


class InvalidTokenError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


class ForbiddenError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


class NotAvailableError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


# client common error below


class ClientKnownError(Error):
    error_type = ErrorType.ClientSideError

    def __init__(self, message: str, **kwargs):
        super().__init__(message=message, **kwargs)


# server common error below


class ServerKnownError(Error):
    error_type = ErrorType.ServerSideError
    message = "A known error has occurred on the service side."

    def __init__(self, cause: str, **kwargs):
        super().__init__(cause=cause, **kwargs)


class ServerUnknownError(Error):
    error_type = ErrorType.ServerSideError
    message = "An unexpected error has occurred on the service side."

    def __init__(self, cause: str, **kwargs):
        super().__init__(cause=cause, **kwargs)
